<?php
namespace Drupal\blocks_by_page_path\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CssCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;

/**
* Route Controller for Blocks by page path
*/
class BlocksByPagePathForm extends FormBase {
  public function getFormId() {
    return 'list_blocks_by_page_path';
  }
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form["path"] = array("#type" => "textfield", 
						  "#title" => t("Page Path / URL Alias"),
						  "#description" => t("Drupal path ex: node/29 or content/services. Note: Wildcard Search not supported Ex: node/*"),
						 );
	$form["submit"] = array("#type" => "submit", 
					"#value" => t("Find"), 
					"#ajax" => array("callback" => array($this, "_get_assigned_blocks"),
								"message" => t("Please wait..."),
								"event" => "click"),
					"#suffix" => '<br><br><div class="output-wrapper"></div>');
	return $form;
  }
  /**
	* Ajax callback to find the blocks which are assigned to particular page.
  */
  public function _get_assigned_blocks(array $form, FormStateInterface $form_state){
	$response = new AjaxResponse();
	// valid drupal paths
	$input_path = $form_state->getValue("path");
	if(!empty($input_path)){
	  $pathValidator = \Drupal::pathValidator()->getUrlIfValid($input_path);
	}else{
	  $input_path = t("empty");
	  $pathValidator = FALSE;
	}
	if($pathValidator){
		$db = \Drupal::database();
		$data = $db->select("config", "c")
				->fields("c")
				->condition("c.name", "block.block%", "LIKE")
				->execute()
				->fetchAll();
		// find the blocks
		$headers = array("Block Name", "Default Theme", "Region", "Configure");
		$rows = array();
		foreach($data as $record){
			$block_details = unserialize($record->data);
			// Conditions
			// 1. Make sure the block is enabled in some regions
			// 2. Make sure the block is assigned to particular page and "Show for the listed pages" settings enables
			if($block_details['region'] != -1){
				if(isset($block_details['visibility']['request_path'])){
					// convert to path alias
					$input_path_alias = \Drupal::service('path_alias.manager')->getAliasByPath("/".$input_path, 'en');
					$pages = !empty($block_details['visibility']['request_path']['pages'])?explode("\n",$block_details['visibility']['request_path']['pages']):array();
					// Trim the data.
				    $pages_included = array();
				    foreach ($pages as $page) {
					  $pages_t = trim($page);
					  $input_path_alias = \Drupal::service('path_alias.manager')->getAliasByPath("/".$pages_t, 'en');
					  $pages_included[] = $input_path_alias;
				    }
				}
				if((isset($block_details['visibility']['request_path']) && empty($block_details['visibility']['request_path']['negate']) && in_array($input_path_alias, $pages_included)) || empty($block_details['visibility'])){
					$block_title = !empty($block_details["settings"]["label"])?$block_details["settings"]["label"]:$record->name;
					// Get route name.
					$path = "admin/structure/block/manage/".$block_details["id"];
					$url_object = \Drupal::service('path.validator')->getUrlIfValid($path);
					$route_name = $url_object->getRouteName();
					// Generate Block Configure URL.
					$url = Url::fromRoute($route_name, array("block" => $block_details["id"]));
					$configure_link = \Drupal::service('link_generator')->generate(t('Configure'), $url);
					
					$rows[]['data'] = array($block_title,
											new FormattableMarkup('<span class="@theme">@theme</b>', ['@theme' => $block_details["theme"]]),
											$block_details["region"],
											$configure_link);
				}
			}
		}
		$output = "";
		$output .= '<div class="blocks-page-path-wrapper">';
		// Filter options by Theme
		$installed_themes = \Drupal::service('extension.list.theme')->getAllInstalledInfo();
		if ($installed_themes) {
			$output .= t('Filter by Theme')."&nbsp;";
			$themes = array_keys($installed_themes);
			$output .= '<select class="blocks-filter-theme">';
			$output .= '<option value="">'.t("-All-").'</option>';
			foreach($themes as $theme) {
				$output .= '<option value= "'.$theme.'">'.$theme.'</option>';
			}
			$output .= '</select>';
		}
		$table = array("#type" => "table",
						"#header" => $headers,
						"#rows" => $rows);
		$output .= \Drupal::service('renderer')->render($table);
		$output .= '</div>';
	}else{
		$output = t("Invalid Path '".$input_path."' provided.");
	}
	// build blocks list
	$response->addCommand(new HtmlCommand('.output-wrapper', $output));
	return $response;
  }
  public function validateForm(array &$form, FormStateInterface $form_state) {
	/*Nothing to validate on this form*/ 
  } 
  public function submitForm(array &$form, FormStateInterface $form_state) {
	/*Nothing to submit on this form*/
  }
}
